<?php

/*
Script to ease the caption editing. 
Requires QDig script ( http://qdig.sourceforge.net/ )

Copyright (C) 2004, 2005, 2006 Ari Takku

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

*/

/*
 * REQUIREMENTS
 * 
 * QDig 1.2.8.x ( * Currently tested with 1.2.8.2)
 * PHP 4.x (should work with PHP 5) 
 * GNU/Linux OS
 * 
 * INSTALLATION
 * 
 * Just copy the captions.php and captions.css to your QDig gallery root. 
 * NB: I use basic HTTP authentication with Apache so BE AWARE that the script
 * does not authenticate users!
 */


/*
 * HISTORY
 * 
 * I was searching a easy to use, online gallery for my digipictures when I
 * found the QDig-gallery script; the year was 2004. I've been a happy user ever
 * since. 
 * 
 * I use the gallery to share the digital images with my friends and relatives.
 * Whereas most of my friends are very capable of using the gallery in English,
 * majority of my relatives are not. So. What I did was just translate
 * the strings in the gallery into Finnish. Said and done. However, when I
 * upgraded the gallery, I had to do the translation (by copy & paste) all over
 * again. I contacted the author of QDig about the issue and he suggested
 * that I could provide the translation to him and that he would include
 * the Finnish translation to the QDig web-site. That's what I call great
 * service! 
 * 
 * There is only one thing which annoys me with QDig - the caption editing. The
 * process of writing tens of captions and constantly saving and browsing
 * and ... aargh. :)  When I take pictures, I want just to quickly to add some
 * captions, and to do it with centralized, easy-to-use manner. I'm not saying
 * the Qdig admin-script is not great - I'm saying this script supports my way
 * of doing the caption editing. :)
 * 
 * I'm happy to hear about comments and suggestions, even though I have only
 * very little spare time compared to those times when this script was written
 * (with great haste and enthusiasm, as one can see from the source:)
 * 
 * 
 * Ari Takku
 */

session_start();

// location of the qdig-specific files
$QFILES = "qdig-files";
$CONVERTED = $QFILES."/converted-images/";
$CAPTIONS = $QFILES."/captions";

// preview pictures name begins with...
$THUMB = "sml_";

// images start with...
$IMG = "IMG_";

// form element names
$GALLERY = "gallery";
$SAVECAPTIONS = "doCaptionsSave";
$MAP = "map_";
$CAPT = "caption_";

// rationalizes the given value (with key 'key') 
// from two arrays: get_arr and sess_arr
function rationalize($key, $initialValue)
{

	$tmp = $initialValue;

	if (isset ($_GET[$key]))
	{
		$tmp = strip_tags($_GET[$key]);
		$_SESSION[$key] = $tmp;
	}

	if (isset ($_SESSION[$key]))
	{
		$tmp = $_SESSION[$key];
	}

	return trim(strip_tags($tmp));
}

// makes the caption first letter capitalized. 
function reformCaption($capt)
{
	return ucfirst(trim($capt));
}

// overwrites given data (string) to a given file
function writeContents($file, $data)
{
	$fp = fopen($file, 'w+');
	fputs($fp, $data);
	fclose($fp);
}

$captionsSavedSuccesfully = false;

// if the captions should be saved
if (isset ($_POST[$SAVECAPTIONS]))
{

	// all files to be saved
	for ($i = 0; true; $i ++)
	{
		if (!isset ($_POST[$MAP.$i]))
		{
			break;
		}

		$file = $CAPTIONS."/".trim(strip_tags($_POST[$MAP.$i]));
		$txt = trim(strip_tags($_POST[$CAPT.$i]));
		writeContents($file, reformCaption($txt));
	}

	$captionsSavedSuccesfully = true;
}

// find all caption files
$d = shell_exec('find -name "*.txt"');

// explode the result so that array contains relative paths
$f_tmp = explode('/'.$CAPTIONS.'/', $d);

// one array for filenames, another for thumbnails (i.e. small-pics), and one for
// directories, which will be used to select what image captions to edit
$files = array ();
$thumbs = array ();
$dirs = array ();

foreach ($f_tmp as $file)
{
	$tmp = explode("\n", $file);
	$file = trim($tmp[0]);

	if ($file == ".")
	{
		continue;
	}

	// save the caption file name
	array_push($files, $file);

	// save the directory
	$tmp = explode($IMG, $file);
	$dirs[$tmp[0]] = "true";

	// save the picture name
	$tmp = explode(".txt", $file);
	$small_pic = str_replace($IMG, $THUMB.$IMG, $tmp[0]);

	array_push($thumbs, $small_pic);
}

// the selected gallery
$galleryArr = array_keys($dirs);
$firstGallery = $galleryArr[count($galleryArr) - 1];
$GLOBALS['gallery'] = rationalize($GALLERY, $firstGallery);

if ($GLOBALS['gallery'] == "" and isset ($_SESSION[$GALLERY]))
{
	$GLOBALS['gallery'] = $_SESSION[$GALLERY];
}
else
{
	$_SESSION[$GALLERY] = $GLOBALS['gallery'];
}

print ('<?xml version="1.0" encoding="ISO-8859-1"?>');
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" 
    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
    <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
 <meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1" />
 <meta http-equiv="Content-Language" content="en" />
 <meta name="Author" content="Ari Takku" />
 <meta name="Description" content="Captionalizer for QDig Image Gallery" />
 <meta name="Robots" content="index,nofollow" />
<title>Captionalizer for QDig Image Gallery</title>

<link href="captions.css" rel="stylesheet" type="text/css" />

<link rel="shortcut icon" href="favicon.png" />

</head>

<body>
<div class="center">
<div class="background">

<?php


if ($captionsSavedSuccesfully)
{
	// notify the user about successfull save
	print "<h3>Captions saved successfully.</h3>";
}
?>



<form method="get" action="captions.php"/>
<fieldset><legend>Selected gallery</legend>
<select name="<?php print $GALLERY; ?>" onchange="javascript:document.forms[0].submit();">

<?php


foreach (array_keys(array_reverse($dirs)) as $gall)
{
	$selected = ($GLOBALS['gallery'] == $gall) ? ' selected="selected"' : "";
	print ("\t<option".$selected.">".$gall."</option>\n");
}
?>

</select>
<input type="submit" value="Select gallery"/>
</fieldset>

</form>

<h1><?php print $GLOBALS['gallery']; ?></h1>

<?php


// function which will be used to filter selected gallery
function filterGallerys($var)
{
	// filtering the results by the chosen gallery  
	if ($GLOBALS['gallery'] != "" and strpos($var, $GLOBALS['gallery']) === false)
	{
		return false;
	}

	return true;
}

$files = array_filter($files, "filterGallerys");
$thumbs = array_filter($thumbs, "filterGallerys");
?>

<form method="POST" enctype="multipart/form-data" action="captions.php">

<?php


$counter = 0;
foreach (array_keys($files) as $i)
{
	$f = $files[$i];
	$t = $thumbs[$i];

	$file_contents = file_get_contents($CAPTIONS.'/'.$f);
	print '<img src="'.$CONVERTED.$t.'" />'."\n";
	print '<input type="hidden" name="'.$MAP.$counter.'" value="'.$f.'" />'."\n";
	print '<textarea name="'.$CAPT.$counter.'">'.$file_contents.'</textarea>'."\n";
	print '<hr />'."\n\n";

	$counter ++;
}
?>

<input type="submit" name="<?php print $SAVECAPTIONS; ?>" value="Save captions"/>


</form>
</div>
</div>

</body>
</html>